/**
 * Copyright 2010 Västra Götalandsregionen
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of version 2.1 of the GNU Lesser General Public
 *   License as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 *   Boston, MA 02111-1307  USA
 */

package se.vgregion.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import java.io.StringReader;
import java.io.StringWriter;

/**
 * Utility class for marshalling and unmarshalling with JAXB.
 * <p/>
 * @author David Rosell
 * @author Patrik Bergström
 */
public class JaxbUtil {
    private static final Logger LOGGER = LoggerFactory.getLogger(JaxbUtil.class);

    private JAXBContext jaxbContext;

    /**
     * Constructor.
     *
     * @param contextNamespace the package name where the JAXB classes are located
     */
    public JaxbUtil(String contextNamespace) throws JAXBException {
        this.jaxbContext = JAXBContext.newInstance(contextNamespace);
    }

    /**
     * Constructor.
     *
     * @param classes the classes which are to be marshalled/unmarshalled
     */
    public JaxbUtil(Class... classes) throws JAXBException {
        this.jaxbContext = JAXBContext.newInstance(classes);
    }

    /**
     * Generic method for unmarshalling an XML <code>String</code> to an <code>Object</code>.
     *
     * @param xml xml
     * @param <T> <code>T</code>
     * @return <code>T</code>
     */
    public <T> T unmarshal(String xml) {
        try {
            //Create marshaller
            Unmarshaller m = jaxbContext.createUnmarshaller();
            //Marshal object into file.
            return (T) m.unmarshal(new StringReader(xml));
        } catch (JAXBException e) {
            throw new RuntimeException("Failed to serialize message", e);
        }
    }

    /**
     * Generic method for marshalling an <code>Object</code> to a <code>String</code>.
     *
     * @param target target
     * @param <T>    <code>T</code>
     * @return <code>T</code>
     */
    public <T> String marshal(T target) {
        StringWriter sw = new StringWriter();
        try {
            //Create marshaller
            Marshaller m = jaxbContext.createMarshaller();
            //Marshal object into file.
            m.marshal(target, sw);
        } catch (JAXBException e) {
            throw new RuntimeException("Failed to serialize message", e);
        }
        return sw.toString();
    }

}
